@@ -9,8 +9,10 @@ from .authenticate import Authenticate
 
 
 class Runner(Authenticate):
-    def __init__(self, asset_name='asset', action_name=None, context=None):
-        # Swimlane
+    def __init__(self, action_name=None, context=None):
+        self.asset_name = None
+        self.action_name = action_name
+        # Swimlane has context
         if context:
             self.asset = context.asset
             self.inputs = context.inputs
@@ -20,12 +22,11 @@ class Runner(Authenticate):
             self.asset_url_key = self.asset_manifest['meta'].get('asset_url_key', 'url')
         # Turbine
         else:
-            self.asset_manifest, self.action_manifest = self._load_schemas(action_name, asset_name)
+            # Data must load before the schemas, we need to determine which schema from the payload.
             self.asset, self.inputs = self._load_data()
-            self.endpoint = self.action_manifest['meta']['endpoint']
-            self.method = self.action_manifest['meta']['method']
-            self.security = self.asset_manifest['meta']['security']
-            self.asset_url_key = self.asset_manifest['meta'].get('asset_url_key', 'url')
+            self.asset_manifest, self.action_manifest = self._load_schemas()
+            self.security = self.asset_manifest.get('meta', {}).get('security', {})
+            self.asset_url_key = self.asset_manifest.get('meta', {}).get('asset_url_key', 'url')
 
         # Build session
         self.session = Session()
@@ -36,11 +37,34 @@ class Runner(Authenticate):
         }
         self.session.verify = os.getenv('verify') or self.asset.get('verify', True)
         self.raise_for_status = os.getenv('raise_for_status') or self.asset.get('raise_for_status', True)
+        self.session.headers.update(self.asset.get('headers', {}))
 
-        super(Runner, self).__init__(session=self.session, asset=self.asset, security=self.security)
+        super(Runner, self).__init__(
+            session=self.session,
+            asset=self.asset,
+            security=self.security,
+            asset_name=self.asset_name
+        )
+
+    def _get_asset_name(self):
+        _in = self.security.get('in')
+        if {'username', 'password'} <= set(self.inputs):
+            return 'http_basic'
+        elif {'token'} <= set(self.inputs):
+            return 'http_bearer'
+        elif {'clientSecret', 'clientId', 'tokenUrl', 'scope'} <= set(self.inputs):
+            return 'oauth2_client_credentials'
+        elif _in == 'header':
+            return 'apikey_header'
+        elif _in == 'query':
+            return 'apikey_query'
+        elif _in == 'cookie':
+            return 'apikey_cookie'
+
+    def _load_schemas(self):
+
+        self.asset_name = self._get_asset_name()
 
-    @staticmethod
-    def _load_schemas(action_name, asset_name):
         manifests = glob.glob('./**/*.yaml', recursive=True)
         asset_manifest = None
         action_manifest = None
@@ -49,10 +73,10 @@ class Runner(Authenticate):
             try:
                 schema = yaml.safe_load(open(manifest).read())
                 schema_type = schema['schema']
-                if schema_type in ['asset/1'] and schema['name'] == asset_name:
+                if schema_type in ['asset/1'] and schema['name'] == self.asset_name:
                     asset_manifest = schema
 
-                if schema_type in ['action/1'] and schema['name'] == action_name:
+                if schema_type in ['action/1'] and schema['name'] == self.action_name:
                     action_manifest = schema
             except:
                 pass
@@ -72,14 +96,20 @@ class Runner(Authenticate):
 
         return asset, inputs
 
-    def get_headers(self):
-        return self.session.headers.update({
+    def _update_headers(self):
+        self.session.headers.update({
             self.inputs.get('headers', {})
         })
+        return
 
     def get_endpoint(self):
         # Mustache if available if not returns string as is.
-        return chevron.render(self.endpoint, self.inputs.get('path_parameters', {}))
+        self.endpoint = chevron.render(self.action_manifest['meta']['endpoint'], self.inputs.get('path_parameters', {}))
+        return self.endpoint
+
+    def get_method(self):
+        self.method = self.action_manifest['meta']['method']
+        return self.method
 
     def get_kwargs(self):
         # todo can we handle files automatically.
@@ -108,9 +138,10 @@ class Runner(Authenticate):
         }
 
     def run(self):
+        self._update_headers()
         response = self.session.request(
-            self.method,
-            urljoin(self.asset[self.asset_url_key], self.get_endpoint()),
+            method=self.get_method(),
+            url=urljoin(self.asset[self.asset_url_key], self.get_endpoint()),
             **self.get_kwargs()
         )
         resp = self.parse_response(response)
